%if 0%{?fedora}
%global with_devel 0
%global with_bundled 1
%global with_debug 1
%else
%global with_devel 0
%global with_bundled 1
%global with_debug 0
%endif

%if 0%{?with_debug}
# https://bugzilla.redhat.com/show_bug.cgi?id=995136#c12
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%global import_path     k8s.io/kubernetes

%global k8s_provider        github
%global k8s_provider_tld    com
%global k8s_project         kubernetes
%global k8s_repo            kubernetes
# https://github.com/kubernetes/kubernetes
%global k8s_provider_prefix %{k8s_provider}.%{k8s_provider_tld}/%{k8s_project}/%{k8s_repo}
# commit picked from openshift/kubernetes although it is available on kubernetes/kubernetes as well
%global k8s_src_dir     Godeps/_workspace/src/k8s.io/kubernetes/
%global k8s_src_dir_sed Godeps\\/_workspace\\/src\\/k8s\\.io\\/kubernetes\\/

#I really need this, otherwise "version_ldflags=$(kube::version_ldflags)"
# does not work
%global _buildshell	/bin/bash
%global _checkshell	/bin/bash

Name:		kubernetes
Version:	1.7.4
Release:	2.0.1%{?dist}
Summary:        Container cluster management
License:        ASL 2.0
Group:          System/Management
URL:           	https://kubernetes.io
Vendor:		Oracle America
ExclusiveArch:  x86_64 ppc64le %{arm} aarch64
Source: 	kubernetes-%{version}.tar.bz2
Source30:       kubernetes-accounting.conf
Source31:       genmanpages.sh
#systemd services
Source10:	kube-apiserver.service
Source11:	kube-controller-manager.service
Source12:	kubelet.service
Source13:	kube-proxy.service
Source14:	kube-scheduler.service
#config files
Source20:	apiserver
Source21:	config
Source22: 	controller-manager
Source23:	kubelet
Source24:	proxy
Source25:	scheduler
Source26:	kubernetes.conf

Source40:	10-kubeadm.conf
Source41: 	kubelet-kubeadm.service
Source42:	kubeadm-setup.sh
Source43:	k8s.conf
#Source44:	kubernetes-dashboard-ol.yaml
Source45:	flannel-ol.yaml
Source46:	kubectl-proxy.service
Source47:	10-kubectl-proxy.conf
#PATCH
#Patch2:         Change-etcd-server-port.patch
#Patch3:         build-with-debug-info.patch
#Patch9:         hack-test-cmd.sh.patch
# Drop apiserver command from hyperkube as apiserver has different permisions and capabilities
# Add kube-prefix for controller-manager, proxy and scheduler
#Patch12:        remove-apiserver-add-kube-prefix-for-hyperkube.patch
#Patch17:        Hyperkube-remove-federation-cmds.patch
# ppc64le
#Patch16:        fix-support-for-ppc64le.patch

# It obsoletes cadvisor but needs its source code (literally integrated)
Obsoletes:      cadvisor

# kubernetes is decomposed into master and node subpackages
# require both of them for updates
Requires: kubernetes-master = %{version}-%{release}
Requires: kubernetes-node = %{version}-%{release}

%description
%{summary}

%package master
Summary: Kubernetes services for master host

BuildRequires: golang >= 1.8
BuildRequires: systemd
BuildRequires: rsync
BuildRequires: go-md2man
BuildRequires: git

Requires(pre): shadow-utils
Requires: kubernetes-client = %{version}-%{release}

# if node is installed with node, version and release must be the same
Conflicts: kubernetes-node < %{version}-%{release}
Conflicts: kubernetes-node > %{version}-%{release}

%description master
Kubernetes services for master host

%package node
Summary: Kubernetes services for node host

Requires: docker-engine >= 1.12.6

BuildRequires: golang >= 1.8
BuildRequires: systemd
BuildRequires: rsync
BuildRequires: go-md2man

Requires(pre): shadow-utils
Requires: socat
Requires: kubernetes-client = %{version}-%{release}

# if master is installed with node, version and release must be the same
Conflicts: kubernetes-master < %{version}-%{release}
Conflicts: kubernetes-master > %{version}-%{release}

%description node
Kubernetes services for node host

%package client
Summary: Kubernetes client tools

BuildRequires: golang >= 1.8
%description client
Kubernetes client tools like kubectl

%package -n kubelet
Summary: Container cluster management
BuildRequires: golang >= 1.8
BuildRequires: systemd
BuildRequires: curl
Requires: iptables >= 1.4.21
Requires: kubernetes-cni = 0.5.2
Requires: socat
Requires: util-linux
Requires: ethtool
Requires: iproute
Requires: ebtables

%description -n kubelet
The node agent of Kubernetes, the container cluster manager.

%package -n kubectl
Summary: Command-line utility for interacting with a Kubernetes cluster.

%description -n kubectl
Command-line utility for interacting with a Kubernetes cluster.

%package -n kubeadm
Summary: Command-line utility for administering a Kubernetes cluster.
Requires: kubelet >= %{version}-%{release}
Requires: kubectl >= %{version}-%{release}
Requires: kubernetes-cni = 0.5.2
Requires: docker-engine >= 1.12.6

%description -n kubeadm
Command-line utility for administering a Kubernetes cluster.

%package -n kubeadm-docker-images
Summary: Contains Oracle built k8s docker images

%description -n kubeadm-docker-images
Contains Oracle built k8s docker images

%prep
%setup -q -n %{k8s_repo}-%{version}

dirs=$(ls | grep -v "^Godeps")

# Move all the code under src/k8s.io/kubernetes directory
mkdir -p src/k8s.io/kubernetes
mv $(ls | grep -v "^src$") src/k8s.io/kubernetes/.

%build
pushd src/k8s.io/kubernetes/
export KUBE_EXTRA_GOPATH=$(pwd)/Godeps/_workspace

export GOPATH=$(pwd)/Godeps/_workspace
export GOBIN=$GOPATH/bin
export PATH=$GOBIN:$PATH
export KUBE_GIT_TREE_STATE=clean

go version

go get -u github.com/jteeuwen/go-bindata/go-bindata

make WHAT='--use_go_build cmd/kube-proxy cmd/kube-apiserver cmd/kube-controller-manager cmd/kubelet cmd/hyperkube plugin/cmd/kube-scheduler cmd/kubectl cmd/kubeadm'

./build-k8s-docker.sh %{version} _output/bin/
# convert md to man
pushd docs
pushd admin
cp kube-apiserver.md kube-controller-manager.md kube-proxy.md kube-scheduler.md kubelet.md ..
popd
cp %{SOURCE31} genmanpages.sh
bash genmanpages.sh
popd
popd

%install
pushd src/k8s.io/kubernetes/
. hack/lib/init.sh
kube::golang::setup_env

%ifarch ppc64le
output_path="${KUBE_OUTPUT_BINPATH}"
%else
output_path="${KUBE_OUTPUT_BINPATH}/$(kube::golang::current_platform)"
%endif

binaries=(kube-apiserver kube-controller-manager kube-scheduler kube-proxy kubelet kubectl hyperkube kubeadm)
install -m 755 -d %{buildroot}%{_bindir}
for bin in "${binaries[@]}"; do
  echo "+++ INSTALLING ${bin}"
  install -p -m 755 -t %{buildroot}%{_bindir} ${output_path}/${bin}
done

# install the bash completion
install -d -m 0755 %{buildroot}%{_datadir}/bash-completion/completions/
%{buildroot}%{_bindir}/kubectl completion bash > %{buildroot}%{_datadir}/bash-completion/completions/kubectl

# install config files
install -d -m 0755 %{buildroot}%{_sysconfdir}/%{name}
#install -m 644 -t %{buildroot}%{_sysconfdir}/%{name} contrib/init/systemd/environ/*
install -m 644 -t %{buildroot}%{_sysconfdir}/%{name} %{SOURCE20}
install -m 644 -t %{buildroot}%{_sysconfdir}/%{name} %{SOURCE21}
install -m 644 -t %{buildroot}%{_sysconfdir}/%{name} %{SOURCE22}
install -m 644 -t %{buildroot}%{_sysconfdir}/%{name} %{SOURCE23}
install -m 644 -t %{buildroot}%{_sysconfdir}/%{name} %{SOURCE24}
install -m 644 -t %{buildroot}%{_sysconfdir}/%{name} %{SOURCE25}

# install service files
install -d -m 0755 %{buildroot}%{_unitdir}
#install -m 0644 -t %{buildroot}%{_unitdir} contrib/init/systemd/*.service
install -m 0644 -t %{buildroot}%{_unitdir}/ %{SOURCE10}
install -m 0644 -t %{buildroot}%{_unitdir}/ %{SOURCE11}
install -m 0644 -t %{buildroot}%{_unitdir}/ %{SOURCE12}
install -m 0644 -t %{buildroot}%{_unitdir}/ %{SOURCE13}
install -m 0644 -t %{buildroot}%{_unitdir}/ %{SOURCE14}

# install manpages
install -d %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/* %{buildroot}%{_mandir}/man1
# from k8s tarball copied docs/man/man1/*.1

# install the place the kubelet defaults to put volumes
install -d %{buildroot}%{_sharedstatedir}/kubelet

# place contrib/init/systemd/tmpfiles.d/kubernetes.conf to /usr/lib/tmpfiles.d/kubernetes.conf
install -d -m 0755 %{buildroot}%{_tmpfilesdir}
#install -p -m 0644 -t %{buildroot}/%{_tmpfilesdir} contrib/init/systemd/tmpfiles.d/kubernetes.conf
install -D -m 0644 %{SOURCE26} %{buildroot}/%{_tmpfilesdir}/kubernetes.conf
mkdir -p %{buildroot}/run
install -d -m 0755 %{buildroot}/run/%{name}/

# enable CPU and Memory accounting
install -d -m 0755 %{buildroot}/%{_sysconfdir}/systemd/system.conf.d
install -p -m 0644 -t %{buildroot}/%{_sysconfdir}/systemd/system.conf.d %{SOURCE30}

# EXTRA FOR KUBEADM/KUBELET
install -m 755 -d %{buildroot}%{_sysconfdir}/cni/net.d/
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d/
install -p -m 755 -t %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d %{SOURCE40}
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/
install -p -m 755 -t %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d %{SOURCE47}

install -m 755 -d %{buildroot}%{_sysconfdir}/kubernetes/manifests/
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/
install -p -m 755 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE41}
install -p -m 755 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE46}
install -p -m 755 -t %{buildroot}%{_bindir} %{SOURCE42}
install -d -m 755 %{buildroot}/etc/sysctl.d
install -p -m 644 -t %{buildroot}/etc/sysctl.d %{SOURCE43}
mv %{buildroot}%{_sysconfdir}/systemd/system/kubelet-kubeadm.service %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service

# EXTRA FOR DOCKER IMAGES
mkdir -p %{buildroot}/usr/local/share/kubeadm
install -m 755 -d %{buildroot}/usr/local/share/kubeadm
images=(kube-apiserver.tar kube-controller-manager.tar kube-scheduler.tar kube-proxy.tar pause-amd64.tar etcd-amd64.tar)
for bin in "${images[@]}"; do
  echo "+++ INSTALLING DOCKER IMAGES ${bin}"
  install -p -m 755 -t %{buildroot}/usr/local/share/kubeadm ${output_path}/oracle_docker/${bin}
done

# INSTALL K8S RELATED YAML
#install -p -m 644 -t %{buildroot}/usr/local/share/kubeadm %{SOURCE44}
install -p -m 644 -t %{buildroot}/usr/local/share/kubeadm %{SOURCE45}

popd
mv src/k8s.io/kubernetes/*.md .
mv src/k8s.io/kubernetes/LICENSE .

%check
#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files
# empty as it depends on master and node

%files master
%license LICENSE
%doc *.md
%{_mandir}/man1/kube-apiserver.1*
%{_mandir}/man1/kube-controller-manager.1*
%{_mandir}/man1/kube-scheduler.1*
%attr(754, -, kube) %caps(cap_net_bind_service=ep) %{_bindir}/kube-apiserver
%{_bindir}/kube-controller-manager
%{_bindir}/kube-scheduler
%{_bindir}/hyperkube
%{_unitdir}/kube-apiserver.service
%{_unitdir}/kube-controller-manager.service
%{_unitdir}/kube-scheduler.service
%dir %{_sysconfdir}/%{name}
%config(noreplace) %{_sysconfdir}/%{name}/apiserver
%config(noreplace) %{_sysconfdir}/%{name}/scheduler
%config(noreplace) %{_sysconfdir}/%{name}/config
%config(noreplace) %{_sysconfdir}/%{name}/controller-manager
%{_tmpfilesdir}/kubernetes.conf
%verify(not size mtime md5) %attr(755, kube,kube) %dir /run/%{name}

%files node
%license LICENSE
%doc *.md
%{_mandir}/man1/kubelet.1*
%{_mandir}/man1/kube-proxy.1*
%{_bindir}/kubelet
%{_bindir}/kube-proxy
%{_bindir}/hyperkube
%{_unitdir}/kube-proxy.service
%{_unitdir}/kubelet.service
%dir %{_sharedstatedir}/kubelet
%dir %{_sysconfdir}/%{name}
%config(noreplace) %{_sysconfdir}/%{name}/config
%config(noreplace) %{_sysconfdir}/%{name}/kubelet
%config(noreplace) %{_sysconfdir}/%{name}/proxy
%config(noreplace) %{_sysconfdir}/systemd/system.conf.d/kubernetes-accounting.conf
%{_tmpfilesdir}/kubernetes.conf
%verify(not size mtime md5) %attr(755, kube,kube) %dir /run/%{name}

%files client
%license LICENSE
%doc *.md
%{_mandir}/man1/kubectl.1*
%{_mandir}/man1/kubectl-*
%{_bindir}/kubectl
%{_bindir}/hyperkube
%{_datadir}/bash-completion/completions/kubectl

%files -n kubelet
%license LICENSE
%{_bindir}/kubelet
%{_sysconfdir}/systemd/system/kubelet.service
%{_sysconfdir}/kubernetes/manifests/

%files -n kubeadm
%license LICENSE
%{_bindir}/kubeadm
%{_sysconfdir}/systemd/system/kubelet.service.d/10-kubeadm.conf
%{_bindir}/kubeadm-setup.sh
/etc/sysctl.d/k8s.conf
#/usr/local/share/kubeadm/kubernetes-dashboard-ol.yaml
/usr/local/share/kubeadm/flannel-ol.yaml

%files -n kubectl
%license LICENSE
%{_bindir}/kubectl
%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/10-kubectl-proxy.conf
%{_sysconfdir}/systemd/system/kubectl-proxy.service

%files -n kubeadm-docker-images
%license LICENSE
/usr/local/share/kubeadm/kube-apiserver.tar
/usr/local/share/kubeadm/kube-controller-manager.tar
/usr/local/share/kubeadm/kube-scheduler.tar
/usr/local/share/kubeadm/kube-proxy.tar
/usr/local/share/kubeadm/pause-amd64.tar
/usr/local/share/kubeadm/etcd-amd64.tar

%pre master
getent group kube >/dev/null || groupadd -r kube
getent passwd kube >/dev/null || useradd -r -g kube -d / -s /sbin/nologin \
        -c "Kubernetes user" kube

%post master
%systemd_post kube-apiserver kube-scheduler kube-controller-manager

%preun master
%systemd_preun kube-apiserver kube-scheduler kube-controller-manager

%postun master
%systemd_postun


%pre node
getent group kube >/dev/null || groupadd -r kube
getent passwd kube >/dev/null || useradd -r -g kube -d / -s /sbin/nologin \
        -c "Kubernetes user" kube

%post node
%systemd_post kubelet kube-proxy
# If accounting is not currently enabled systemd reexec
if [[ `systemctl show docker kubelet | grep -q -e CPUAccounting=no -e MemoryAccounting=no; echo $?` -eq 0 ]]; then
  systemctl daemon-reexec
fi

%preun node
%systemd_preun kubelet kube-proxy

%postun node
%systemd_postun

%post -n kubelet
if [ $1 -gt 1 ] ; then
  systemctl daemon-reload >/dev/null 2>&1
  systemctl try-restart kubelet >/dev/null 2>&1
fi

%preun -n kubelet
%systemd_preun kubelet

%postun -n kubelet
%systemd_postun_with_restart kubelet

%changelog
* Wed Sep 06 2017 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.7.4-2.0.1
- Update to v1.7.4
- [Orabug 26677088] kube-dns failure with iptables services

* Tue Aug 01 2017 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.6.4-2.0.1
- Update to v1.6.4
- Include kubeadm-setup.sh for ease of provisioning via kubeadm with Oracle Linux
